"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const flatten_1 = __importDefault(require("lodash/flatten"));
const map_1 = __importDefault(require("lodash/map"));
const omit_1 = __importDefault(require("lodash/omit"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const rc_config_loader_1 = require("rc-config-loader");
const cli_options_1 = require("../cli-options");
const programError_1 = __importDefault(require("./programError"));
/** Loads the .ncurc config file. */
async function getNcuRc({ configFileName, configFilePath, packageFile, global, options, }) {
    const { default: chalkDefault, Chalk } = await import('chalk');
    const chalk = options?.color ? new Chalk({ level: 1 }) : chalkDefault;
    const rawResult = (0, rc_config_loader_1.rcFile)('ncurc', {
        configFileName: configFileName || '.ncurc',
        defaultExtension: ['.json', '.yml', '.js'],
        cwd: configFilePath || (global ? os_1.default.homedir() : packageFile ? path_1.default.dirname(packageFile) : undefined),
    });
    if (configFileName && !rawResult?.filePath) {
        (0, programError_1.default)(options, `Config file ${configFileName} not found in ${configFilePath || process.cwd()}`);
    }
    const result = {
        filePath: rawResult?.filePath,
        // Prevent the cli tool from choking because of an unknown option "$schema"
        config: (0, omit_1.default)(rawResult?.config, '$schema'),
    };
    // validate arguments here to provide a better error message
    const unknownOptions = Object.keys(result?.config || {}).filter(arg => !cli_options_1.cliOptionsMap[arg]);
    if (unknownOptions.length > 0) {
        console.error(chalk.red(`Unknown option${unknownOptions.length === 1 ? '' : 's'} found in config file:`), chalk.gray(unknownOptions.join(', ')));
        console.info('Using config file ' + result.filePath);
        console.info(`You can change the config file path with ${chalk.blue('--configFilePath')}`);
    }
    // flatten config object into command line arguments to be read by commander
    const args = result
        ? (0, flatten_1.default)((0, map_1.default)(result.config, (value, name) => 
        // if a boolean option is true, include only the nullary option --${name}
        // an option is considered boolean if its type is explicitly set to boolean, or if it is has a proper Javascript boolean value
        value === true || (cli_options_1.cliOptionsMap[name]?.type === 'boolean' && value)
            ? [`--${name}`]
            : // if a boolean option is false, exclude it
                value === false || (cli_options_1.cliOptionsMap[name]?.type === 'boolean' && !value)
                    ? []
                    : // otherwise render as a 2-tuple
                        [`--${name}`, value]))
        : [];
    return result ? { ...result, args } : null;
}
exports.default = getNcuRc;
//# sourceMappingURL=getNcuRc.js.map